@@ -36,7 +36,7 @@ public:
     ~Pty();
 
     bool run(QString pgm, QStringList args);
-    bool writeCmd(QString* message, QString* response, int msecTimeout);
+    bool writeCmd(QString* message, QString* response, int msecTimeout, QString* endString=0);
 
 private:
     int m_Fd;
