@@ -29,6 +29,7 @@ SmsSender::SmsSender(int argc, char *argv[]) :
     m_Db = NULL;
     m_LogFile = NULL;
     m_Save = true;
+    m_Send = false;
     m_Argc = argc - 1;
     for (int i=1; i<argc; i++)
         m_Argv.append(argv[i]);
@@ -64,11 +65,12 @@ void SmsSender::printUsage()
     consoleOutput("Usage:");
     consoleOutput("  smssend [options] -n +39xxxxxx:+39xxxxxx -m \"Text\"");
     consoleOutput("Options:");
-    consoleOutput("  -c         : compose message with conversation");
+    consoleOutput("  -c         : compose the message with conversation");
+    consoleOutput("  -C         : send the message with conversation");
     consoleOutput("  -l logfile : log pnatd messages to logfile");
-    consoleOutput("               This option is useless if used with -c");
+    consoleOutput("               This option is useless if used with -c or -C");
     consoleOutput("  -s         : don't save message in the sent folder");
-    consoleOutput("               This option is useless if used with -c");
+    consoleOutput("               This option is useless if used with -c or -C");
 
 }
 
@@ -98,7 +100,10 @@ int SmsSender::exec()
                 currArg = SmsSender::argLog;
             else if (param == "-c")
                 m_Compose = true;
-            else if (param == "-s")
+            else if (param == "-C"){
+                m_Compose = true;
+                m_Send = true;
+            }else if (param == "-s")
                 m_Save = false;
             else{
                 consoleOutput(QString("ERR: unrecognized argument: %1").arg(param));
@@ -107,9 +112,16 @@ int SmsSender::exec()
         }else{
             switch (currArg){
                 case SmsSender::argNumber:
-                    foreach (QString num, param.split(":")){
-                        if (!numbers.contains(num))
-                            numbers.append(num);
+                    {
+                        QRegExp exp("\\+?[0-9]*");
+                        foreach (QString num, param.split(":")){
+                            if (!exp.exactMatch(num)){
+                                consoleOutput(QString("WARNING: Invalid receiver number %1").arg(num));
+                                continue;
+                            }
+                            if (!numbers.contains(num))
+                                numbers.append(num);
+                        }
                     }
                     break;
                 case SmsSender::argText:
@@ -152,6 +164,10 @@ int SmsSender::exec()
             consoleOutput("WARNING: error opening events database");
             delete m_Db;
             m_Db = NULL;
+        }else{
+            QSqlQuery qry;
+            if (qry.prepare("PRAGMA synchronous=OFF;"))
+                qry.exec();
         }
 
         QStringList messageBlocks;
@@ -173,12 +189,6 @@ int SmsSender::exec()
         int i = 0;
         foreach(QString number, numbers){
             i++;
-            QRegExp exp("\\+?[0-9]*");
-            if (!exp.exactMatch(number)){
-                consoleOutput(QString("ERR: Invalid receiver number %1").arg(number));
-                continue;
-            }
-
             consoleOutput(QString("[%1/%2]").arg(i).arg(totalMsg), false);
             foreach (QString msg, messageBlocks){
                 if (m_Compose)
@@ -206,12 +216,18 @@ bool SmsSender::compose(QString number, QString messageText)
     message.setTo(QMessageAddress(QMessageAddress::Phone, number));
     message.setBody(messageText);
 
-    if (service.compose(message)){
-        consoleOutput(".");
-        //consoleOutput(QString("OK: sms queued for sending to %1").arg(number));
-        return true;
+    if (m_Send){
+        if (service.send(message)){
+            consoleOutput(".");
+            consoleOutput(QString("OK: sms queued for sending to %1").arg(number));
+            return true;
+        }
+    }else{
+        if (service.compose(message)){
+            consoleOutput(".");
+            return true;
+        }
     }
-
     consoleOutput(QString("ERR: Failed to queue sms for sending to %1"));
     return false;
 }
@@ -252,7 +268,7 @@ bool SmsSender::send(QString number, QString messageText)
     }
 
     if (res){
-        if (!writeCmd(pty, messageText, "")){
+        if (!writeCmd(pty, messageText, messageText)){
             consoleOutput(QString("\nERR: Failed to send text message"));
             res = false;
         }else
@@ -288,8 +304,6 @@ bool SmsSender::writeCmd(Pty* p, QString cmd, QString okReply, int msectimeout)
     if (m_LogFile != NULL)
         m_LogFile->write(fullRes.toLatin1());
 
-    //consoleOutput(QString("INFO: pry reply \"%1\"").arg(fullRes));
-
     if (!okReply.isEmpty()){
         QStringList lines = fullRes.split(QRegExp("\r|\r\n"), QString::SkipEmptyParts);
         QString res = lines.last().trimmed();
@@ -308,18 +322,31 @@ bool SmsSender::saveMsgToSentFolder(QString number, QString message)
     //ATM QtMobility doesn't have a method to save a message in a folder.
     message.replace("'", "''");
     QString partNumber = number.right(7);
+
+    QSqlQuery qry;
+    if (qry.prepare("BEGIN IMMEDIATE TRANSACTION;"))
+        qry.exec();
+
     QString sql = QString("insert into Events (service_id, event_type_id, storage_time, start_time, end_time, is_read, flags, \
                                                bytes_sent, bytes_received, local_uid, local_name, remote_uid, free_text, group_uid, outgoing) \
                            values (3, 7, strftime('%s', 'now'), strftime('%s', 'now'), 0, 1, 0, 0, 0, 'ring/tel/ring', '<SelfHandle>', '%1', '%2', '%3', 1)")
                            .arg(number)
                            .arg(message)
                            .arg(partNumber);
-    QSqlQuery qry;
-    if (!qry.prepare(sql))
+    if (!qry.prepare(sql)){
+        if (qry.prepare("ROLLBACK;"))
+            qry.exec();
         return false;
+    }
 
-    if (!qry.exec())
+    if (!qry.exec()){
+        if (qry.prepare("ROLLBACK;"))
+            qry.exec();
         return false;
+    }
+
+    if (qry.prepare("COMMIT;"))
+        qry.exec();
 
     //QMessageService service;
     //service.exportUpdates(QMessageAccount::defaultAccount(QMessage::Sms));
